/* 
    Copyright 2015 Rodrigo Fernandes da Costa

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Author: Rodrigo Fernandes da Costa
    E-mail: rodrigo5244@gmail.com
*/

var path = {};

(function () {
    "use strict";

    if (window.navigator.platform === "Win32") {
        path.separator = "\\";
    } else {
        path.separator = "/";
    }
}());

path.folder = function (self) {
    "use strict";

    self = self || {};

    if (self.path.slice(-1) !== path.separator) {
        throw "File path, not folder path";
    }

    self.type = "folder";

    self.getEntries = function (callback) {
        var list = [];

        exposer.api("folder/get", function (entries) {
            if (entries) {
                entries = entries.split("\n");

                entries.forEach(function (entry) {
                    try {
                        list.push(path.folder({path: self.path + entry}));
                    } catch (exception) {
                        if (typeof exception !== "string") {
                            throw exception;
                        }

                        list.push(path.file({path: self.path + entry}));
                    }
                });
            }

            callback(list);
        }, self.path);
    };

    self.getAllEntries = function (callback) {
        var pending, allEntries, recurse;

        pending = 0;
        allEntries = [];

        recurse = function (entries) {
            pending = pending - 1;

            entries.forEach(function (entry) {
                allEntries.push(entry);

                if (entry.getAllEntries) {
                    pending = pending + 1;
                    entry.getAllEntries(recurse);
                }
            });

            if (pending === 0) {
                if (callback) {
                    callback(allEntries);
                }
            }
        };

        pending = pending + 1;
        self.getEntries(recurse);
    };

    self.remove = function (callback) {
        self.getEntries(function (entries) {
            if (entries.length) {
                entries.forEach(function (entry) {
                    entry.remove(function () {
                        entries.length = entries.length - 1;
                        if (entries.length === 0) {
                            exposer.api("folder/remove", function () {
                                if (callback) {
                                    callback();
                                }
                            }, self.path);
                        }
                    });
                });
            } else {
                exposer.api("folder/remove", function () {
                    if (callback) {
                        callback();
                    }
                }, self.path);
            }
        });
    };

    self.toString = function () {
        return self.path;
    };

    return self;
};

path.file = function (self) {
    "use strict";

    self = self || {};

    if (self.path.slice(-1) === path.separator) {
        throw "Folder path, not file path";
    }

    self.type = "file";

    self.name = self.path.split(path.separator).slice(-1)[0];
    self.extention = self.name.split(".").slice(-1)[0];
    if (self.extention === self.name) {
        self.extention = "";
    }

    self.remove = function (callback) {
        exposer.api("file/remove", callback, self.path);

        return self;
    };

    self.toString = function () {
        return self.path;
    };

    self.rename = function (fresh, callback) {
        exposer.api("file/rename", callback, self.path, fresh);
        self.path = fresh;
    };

    return self;
};
