/* 
    Copyright 2015 Rodrigo Fernandes da Costa

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Author: Rodrigo Fernandes da Costa
    E-mail: rodrigo5244@gmail.com
*/

var extentions = ["", "css", "svg", "xml"],
    patterns = [
        /#[0-9A-F]+/ig,
        /((rgba?)|(hsla?))\s*\(\s*\d+\s*(,\s*[\.0-9]+\s*){2,3}\)/ig
    ];

function write(archive, card) {
    "use strict";

    var pending;

    pending = 0;

    Object.keys(archive).forEach(function (path) {
        pending = pending + 1;
        exposer.api("file/set", function () {
            pending = pending - 1;

            if (pending === 0) {
                card.tag.innerHTML = "<h2>Done!</h2>";
            }
        }, path, archive[path]);
    });
}

function show(archive, colors) {
    "use strict";

    var card, table, row, cell, button, map;

    map = function () {
        Object.keys(archive).forEach(function (path) {
            Object.keys(colors).forEach(function (color) {
                var pattern;

                if (!colors[color]) {
                    return;
                }

                pattern = new RegExp(color, "ig");

                archive[path] = archive[path].replace(pattern, colors[color]);
            });
        });

        write(archive, card);
    };

    table = window.document.createElement("table");

    row = table.insertRow();
    row.insertCell().innerHTML = "From";
    row.insertCell().innerHTML = "To";

    Object.keys(colors).forEach(function (colorString) {
        var input, color;

        row = table.insertRow();

        input = window.document.createElement("input");
        input.className = "color";
        input.value = colorString;
        color = new jscolor.color(input);
        color.pickerOnfocus = false;

        cell = row.insertCell();
        cell.appendChild(input);

        input = window.document.createElement("input");
        input.className = "color";
        input.value = colorString;
        color = new jscolor.color(input);

        cell = row.insertCell();
        cell.appendChild(input);

        input.addEventListener("change", function () {
            colors[colorString] = "#" + input.value;
        });
    });

    card = ui.card({title: "Color map"});
    card.tag.appendChild(table);
    ui.cards.appendChild(card.tag);

    card.tag.appendChild(window.document.createElement("br"));

    button = window.document.createElement("button");
    button.innerHTML = "Convert";
    button.addEventListener("click", map);
    card.tag.appendChild(button);
}

function parse(archive) {
    "use strict";

    var colors = {};

    Object.keys(archive).forEach(function (path) {
        patterns.forEach(function (pattern) {
            var matches = archive[path].match(pattern) || [];

            matches.forEach(function (color) {
                colors[color] = false;
            });
        });
    });

    show(archive, colors);
}

function load(files) {
    "use strict";

    var pending, archive, card;

    files = files.filter(function (file) {
        return extentions.indexOf(file.extention) !== -1;
    });

    if (files.length === 0) {
        card = ui.card({title: "No colors"});
        ui.cards.appendChild(card.tag);
        return;
    }

    archive = {};
    pending = 0;

    files.forEach(function (file) {

        pending = pending + 1;
        exposer.api("file/get", function (string) {
            pending = pending - 1;

            archive[file.path] = string;

            if (pending === 0) {
                parse(archive);
            }
        }, file.path);
    });
}

function list(paths) {
    "use strict";

    var all, pending;

    pending = 0;
    all = [];

    paths.forEach(function (listPath) {
        try {
            all.push(path.file({path: listPath}));
        } catch (exception) {
            if (typeof exception !== "string") {
                throw exception;
            }

            pending = pending + 1;
            path.folder({path: listPath}).getAllEntries(function (entries) {
                pending = pending - 1;

                all = all.concat(entries);

                if (pending === 0) {
                    load(all);
                }
            });
        }
    });

    if (pending === 0) {
        load(all);
    }
}

function paths() {
    var p, inputs, addInput, button;

    inputs = [];

    addInput = function () {
        var input;

        input = window.document.createElement("input");
        input.size = 20;
        input.type = "text";
        input.style.fontFamily = "Courier";
        input.addEventListener("change", function () {
            input.size = Math.min(Math.max(input.value.length, 20), 100);

            if (input === inputs[inputs.length - 1]) {
                addInput();
            }
        });

        inputs.push(input);

        card.tag.insertBefore(input, button);
        card.tag.insertBefore(window.document.createElement("br"), button);
        card.tag.insertBefore(window.document.createElement("br"), button);
    };

    card = ui.card({title: "No command line arguments"});
    ui.cards.appendChild(card.tag);

    p = window.document.createElement("p");
    p.innerHTML = "Type the path of the files or folders.";
    card.tag.appendChild(p);

    button = window.document.createElement("button");
    button.addEventListener("click", function () {
        ui.cards.removeChild(card.tag);

        list(inputs.filter(function (input) {
            return input.value;
        }).map(function (input) {
            return input.value;
        }));
    });
    button.innerHTML = "Load";
    card.tag.appendChild(button);

    addInput();
}

window.addEventListener("load", function () {
    "use strict";

    var card;

    exposer.api("argv", function (argv) {
        argv = argv.split("\n").filter(function (arg) {
            return arg[0] !== "-";
        });

        if (argv.length === 1) {
            paths();
            return;
        }

        list(argv.slice(1));
    });
});
